import java.awt.*;


public class DoubleLineGraph extends Graph {

	Float xCoords2[];				// Second set of coordinates
	Float yCoords2[];				
	Point pixels2[];				// pixel points for second values
	int length2;

	public DoubleLineGraph() {
		super();
		length2 = 0;
		xCoords2 = new Float[1];
		xCoords2[0] = new Float(0);
		yCoords2 = new Float[1];
		yCoords2[0] = new Float(0);
	}
	
	public DoubleLineGraph(Float[] x1, Float[] y1, Float[] x2, Float[] y2) {
		super(x1, y1);
		length2 = x2.length;
		xCoords2 = new Float[length2];
		yCoords2 = new Float[length2];
		for(int i = 0; i < length2; i++)		// Copy arrays, and store range variables
		{
			xCoords2[i] = x2[i];
			yCoords2[i] = y2[i];
			if (xCoords2[i].floatValue() > maxX)
				maxX = xCoords2[i].floatValue();
			if (xCoords2[i].floatValue() < minX)
				minX = xCoords2[i].floatValue();
			if (yCoords2[i].floatValue() > maxY)
				maxY = yCoords2[i].floatValue();
			if (yCoords2[i].floatValue() < minY)
				minY = yCoords2[i].floatValue();
		}
	}
	
	// Determines size of this object, and scales the coordinates accordingly.
	public void setPoints() {
		super.setPoints();
		
		pixels2 = new Point[length2];			// Initialize array for actual points to be graphed
		
		int x, y;									// current x and y coordinates
		for (int i = 0; i < length2; i++) {
			// Calculate what the coordinates of each data point should be, then add a Point object to the array
			x = adjX(xCoords2[i]);
			y = adjY(yCoords2[i]);
			pixels2[i] = new Point(x, y);
		}
	}
	
	public void paint(Graphics g) {
		Font f;
		String fName;
		FontMetrics fm;
	
		if (length > 0) { 					// Make sure we have at least one point
			f = g.getFont();
			fName = f.getName();
			f = new Font(fName, Font.PLAIN, 9);
			g.setFont(f);
			fm = g.getFontMetrics();
		
			setPoints();				// When we are ready to paint, the dimensions of the component have been
			setTicks(false, fm);			// set, so we can calculate coordinates of all the points
			
			g.setColor(new Color(150, 0, 0));
			for (int i = 0; i < length - 1; i++) {		// For each point
				g.drawLine(pixels[i].x, pixels[i].y, pixels[i + 1].x,
					pixels[i + 1].y);				// Connect it to the next point
			}
			
			g.setColor(new Color(0, 0, 100));	
			for (int i = 0; i < length2 - 1; i++) {		// For each point
				g.drawLine(pixels2[i].x, pixels2[i].y, pixels2[i + 1].x,
					pixels2[i + 1].y);				// Connect it to the next point
			}
			
			g.setColor(new Color(0, 0, 0));
			drawXAxis(g);
			drawYAxis(g);
			g.setColor(new Color(150, 0, 0));			// Make ticks red
			drawTicks(g);
			g.setColor(new Color(0, 0, 100));			// Make labels blue
			labelTicks(g);
			g.setColor(new Color(0, 0, 0));
		}
	}	
}