import java.awt.*;


public class LineSpikeGraph extends Graph {

	Float xSpikes[];				// Second set of coordinates
	Float ySpikes[];				
	Point pixelsSpike[];				// pixel points for second values
	int lengthSpike;
	boolean hiliteOn;
	float beginHilite, endHilite;

	public LineSpikeGraph() {
		super();
		lengthSpike = 0;
		xSpikes = new Float[1];
		xSpikes[0] = new Float(0);
		ySpikes = new Float[1];
		ySpikes[0] = new Float(0);
		hiliteOn = false;
	}
	
	public LineSpikeGraph(Float[] xl, Float[] yl, Float[] xs, Float[] ys) {
		super(xl, yl);
		lengthSpike = xs.length;
		xSpikes = new Float[lengthSpike];
		ySpikes = new Float[lengthSpike];
		for(int i = 0; i < lengthSpike; i++)		// Copy arrays, and store range variables
		{
			xSpikes[i] = xs[i];
			ySpikes[i] = ys[i];
			if (xSpikes[i].floatValue() > maxX)
				maxX = xSpikes[i].floatValue();
			if (xSpikes[i].floatValue() < minX)
				minX = xSpikes[i].floatValue();
			if (ySpikes[i].floatValue() > maxY)
				maxY = ySpikes[i].floatValue();
			if (ySpikes[i].floatValue() < minY)
				minY = ySpikes[i].floatValue();
		}
		hiliteOn = false;
	}
	
	public LineSpikeGraph(Float[] xl, Float[] yl, Float[] xs, Float[] ys, float bh, float eh) {
		super(xl, yl);
		lengthSpike = xs.length;
		xSpikes = new Float[lengthSpike];
		ySpikes = new Float[lengthSpike];
		for(int i = 0; i < lengthSpike; i++)		// Copy arrays, and store range variables
		{
			xSpikes[i] = xs[i];
			ySpikes[i] = ys[i];
			if (xSpikes[i].floatValue() > maxX)
				maxX = xSpikes[i].floatValue();
			if (xSpikes[i].floatValue() < minX)
				minX = xSpikes[i].floatValue();
			if (ySpikes[i].floatValue() > maxY)
				maxY = ySpikes[i].floatValue();
			if (ySpikes[i].floatValue() < minY)
				minY = ySpikes[i].floatValue();
		}
		beginHilite = bh;
		endHilite = eh;
		hiliteOn = true;
	}
	
	// Determines size of this object, and scales the coordinates accordingly.
	public void setPoints() {
		super.setPoints();
		
		pixelsSpike = new Point[lengthSpike];			// Initialize array for actual points to be graphed
		
		int x, y;									// current x and y coordinates
		for (int i = 0; i < lengthSpike; i++) {
			// Calculate what the coordinates of each data point should be, then add a Point object to the array
			x = adjX(xSpikes[i]);
			y = adjY(ySpikes[i]);
			pixelsSpike[i] = new Point(x, y);
		}
	}
	
	public void paint(Graphics g) {
		Font f;
		String fName;
		FontMetrics fm;
	
		if (length > 0) { 					// Make sure we have at least one point
			f = g.getFont();
			fName = f.getName();
			f = new Font(fName, Font.PLAIN, 9);
			g.setFont(f);
			fm = g.getFontMetrics();
		
			setPoints();				// When we are ready to paint, the dimensions of the component have been
			setTicks(false, fm);			// set, so we can calculate coordinates of all the points
			
			g.setColor(new Color(150, 0, 0));
			for (int i = 0; i < length - 1; i++) {		// For each point
				g.drawLine(pixels[i].x, pixels[i].y, pixels[i + 1].x,
					pixels[i + 1].y);				// Connect it to the next point
			}
			
			for (int i = 0; i < lengthSpike; i++) {		// For each point
				if (hiliteOn == true && xSpikes[i].floatValue() >= beginHilite && 
						xSpikes[i].floatValue() <= endHilite) {
					g.setColor(new Color(100, 0, 0));
					g.drawLine(pixelsSpike[i].x, adjY(0), pixelsSpike[i].x, pixelsSpike[i].y);
					g.drawLine(pixelsSpike[i].x + 1, adjY(0), pixelsSpike[i].x + 1, pixelsSpike[i].y);
				}
				else {
					g.setColor(new Color(0, 0, 0));
					g.drawLine(pixelsSpike[i].x, adjY(0), pixelsSpike[i].x, pixelsSpike[i].y);
				}
			}
			
			g.setColor(new Color(0, 0, 0));
			drawXAxis(g);
			g.setColor(new Color(150, 0, 0));			// Make ticks red
			drawTicks(g);
			g.setColor(new Color(0, 0, 100));			// Make labels blue
			labelTicks(g);
			g.setColor(new Color(0, 0, 0));
		}
	}	
}