// NeedleArea.java

import java.awt.*;

public class NeedleArea extends java.awt.Canvas {
	
	int w, h;								// width and height of this component

	Point needleStarts[];				// One end of each needle
	Point needleEnds[];				// The other end of the corresponding needle
	
	float estimate;					// Estimate of pi
	
	int num;						// Number of data points
	JRandom myRand;
	
	// Set up a NeedleArea, and calculate the needle positions
	public NeedleArea(int n) {
		super();
		num = n;
		
		needleStarts = new Point[0];
		needleEnds = new Point[0];
		
		myRand = new JRandom();
	}
	
	
	// Randomly throws the needles
	public float setPoints() {
		needleStarts = new Point[num];
		needleEnds = new Point[num];
	
		Dimension d = size();  //getSize() in 1.1.1
		w = d.width;				
		h = d.height;	
		
		float y;									// y coordinates, relative to board ( 0 - 1 )
		float angle;								// angle of needle
		int crosses = 0;							// How many cross a crack
		
		
		for (int i = 0; i < num; i++) {
			// Choose random positioning of each needle, then compute and add start and end points to arrays
		
			y = myRand.nextFloat(1);
			angle = myRand.nextFloat(2f * (float) Math.PI);
			
			needleStarts[i] = new Point(myRand.nextInt(w - 40) + 20, 
				(int) (20f * (y + myRand.nextInt((int) Math.floor(d.height / 20) - 2))));
			needleEnds[i] = coordsOf(needleStarts[i], angle, 20);
			if ((y - Math.sin(angle)) > 1 || (y - Math.sin(angle)) <= 0)
				crosses++;
		}
		estimate = 2f / ((float) crosses / (float) num);
		estimate = Math.round(100000f * estimate) / 100000f;
		return estimate;
	}
	
	public Point coordsOf(Point start, float angle, float distance) {
		int x, y;
		x = start.x + (int) (Math.cos(angle) * distance);
		y = start.y - (int) (Math.sin(angle) * distance);
		return new Point(x, y);
	}
	
	// Paint method - calculates and paints lines, sets up needles, and paints them
	public void paint(Graphics g) {
		if (num > 0) { 					// Make sure we have at least one point
			//setPoints();				// When we are ready to paint, the dimensions of the component have been
			//g.setColor(new Color(0, 0, 0));
			drawLines(g);
			drawNeedles(g);
		}
	}
	
	public void drawLines(Graphics g) {
		for (int i = 0; i <= h + 20; i += 20) {
			g.drawLine(0, i, w, i);
		}
	}
	
	public void drawNeedles(Graphics g) {
		for (int i = 0; i < needleStarts.length; i++) {
		g.drawLine(needleStarts[i].x, needleStarts[i].y, needleEnds[i].x, needleEnds[i].y);
		}
	}
}