// CLTBernoulli.java

// Written by Julian Devlin, 8/97, for the text book
// "Introduction to Probability," by Charles M. Grinstead & J. Laurie Snell

// Doesn't work for n > 400

import java.applet.Applet;
import java.awt.*;

public class CLTBernoulli
	extends java.applet.Applet
{
	Float[] xSpikes;		// Variables for simulation
	Float[] ySpikes;
	Float[] xLines;
	Float[] yLines;
	
	LineSpikeGraph lsg;				// AWT elements
	
	Panel dispArea;
	Panel controls;		// Panel for user controls
	
	Label numl1, numl2, numl3, numl4, numl5, numl6, numl7, numl8;			// Controls
	TextField num1, num2, num3, num4;
	Button go;
	
	GridBagLayout gbl;
	GridBagConstraints cc;
	
	float mean, sigma;
		
	// Initialize applet
	public void init()
	{	
		numl1 = new Label("n =");			// Create controls
		num1 = new TextField("100", 4);
		numl2 = new Label("p =");			// Create controls
		num2 = new TextField(".5", 4);
		numl3 = new Label("kmin =");			// Create controls
		num3 = new TextField("", 4);
		numl4 = new Label("kmax =");			// Create controls
		num4 = new TextField("", 4);
		numl5 = new Label("exact =");
		numl6 = new Label("                  ");
		numl7 = new Label("approx =");
		numl8 = new Label("                  ");
		go = new Button("Go");
		
		lsg = new LineSpikeGraph(); // initialize a graphing space
		
		dispArea = new Panel();				// Set up window
		controls = new Panel();
		setLayout(new BorderLayout(5, 5));
		
		add("South", controls);
		add("Center", dispArea);
		
		dispArea.setLayout(new GridLayout(1, 1));
		dispArea.add(lsg);
		
		gbl = new GridBagLayout();
		controls.setLayout(gbl);
		
		cc = new GridBagConstraints();
		
		cc.gridx = 0;
		cc.gridy = 0;
		gbl.setConstraints(numl1, cc);
		controls.add(numl1);
		
		cc.gridx = 1;
		gbl.setConstraints(num1, cc);
		controls.add(num1);
		
		cc.gridx = 2;
		gbl.setConstraints(numl2, cc);
		controls.add(numl2);
		
		cc.gridx = 3;
		gbl.setConstraints(num2, cc);
		controls.add(num2);
		
		cc.gridx = 0;
		cc.gridy = 1;
		gbl.setConstraints(numl3, cc);
		controls.add(numl3);
		
		cc.gridx = 1;
		gbl.setConstraints(num3, cc);
		controls.add(num3);
		
		cc.gridx = 2;
		gbl.setConstraints(numl4, cc);
		controls.add(numl4);
		
		cc.gridx = 3;
		gbl.setConstraints(num4, cc);
		controls.add(num4);
		
		cc.gridx = 0;
		cc.gridy = 2;
		gbl.setConstraints(numl5, cc);
		controls.add(numl5);
		
		cc.gridx = 1;
		gbl.setConstraints(numl6, cc);
		controls.add(numl6);
		
		cc.gridx = 2;
		gbl.setConstraints(numl7, cc);
		controls.add(numl7);
		
		cc.gridx = 3;
		gbl.setConstraints(numl8, cc);
		controls.add(numl8);
		
		cc.gridx = 0;
		cc.gridy = 3;
		cc.gridwidth = 4;
		gbl.setConstraints(go, cc);
		controls.add(go);
		
		validate();
	}
	
	// Handle events
	public boolean handleEvent(Event evt)
	{
		String minStr, maxStr;
		if (evt.target instanceof Button)
		{
			if (evt.target == go && evt.id == Event.ACTION_EVENT)	// When button is clicked
			{
				minStr = num3.getText().trim();
				maxStr = num4.getText().trim();
				if (minStr.length() != 0 && maxStr.length() != 0)
					simulate(Integer.valueOf(num1.getText()).intValue(),
        				Float.valueOf(num2.getText()).floatValue(),
        				Integer.valueOf(minStr).intValue(),
        				Integer.valueOf(maxStr).intValue());
        		else if (minStr.length() != 0)
        			simulate(Integer.valueOf(num1.getText()).intValue(),
        				Float.valueOf(num2.getText()).floatValue(),
        				Integer.valueOf(minStr).intValue(),
        				Integer.valueOf(maxStr).intValue());
        		else
        			simulate(Integer.valueOf(num1.getText()).intValue(),
        				Float.valueOf(num2.getText()).floatValue());
        		return true;					// Generate correct number of tosses
			}
		}
		return super.handleEvent(evt);	// Handle other events as usual
	}
	
	public float normal(float x, float mu, float sigma) {
		return 1 / (sigma * (float)  Math.pow(2 * Math.PI, .5)) * 
			(float) Math.pow(Math.E, -1 * Math.pow(x - mu, 2) / (2 * 
				Math.pow((double) sigma, 2)));	
	}
	
	public float normalArea(float a, float b) {
		float mu = 0f;
		float sigma = 1f;
		int subdivisions = (int) Math.max(100, 20 * (double) Math.round(
			(double) b - (double) a + .5));
		float dx = (b - a) / (float) subdivisions;
		float sum = normal(a, mu, sigma) + normal(b, mu, sigma);
		float x;
		for (int k = 1; k < subdivisions; k++) {
			x = a + (float) k * dx;
			if (k % 2 == 1)
				sum += 4 * normal(x, mu, sigma);
			else
				sum += 2 * normal(x, mu, sigma);
		}
		return dx / 3 * sum;
	}
	
	public void setPoints(int num, float prob) 
	{
		float temp;
		mean = (float) num * prob;
    	sigma = (float) Math.pow((double) ((float) num * prob 
    		* (1 - prob)), .5);
    	float xmin = -4f;
    	float xmax = 4f;
    	float dx = (xmax - xmin) / 100f;
    	int kmin = (int) Math.floor(Math.max(0, (double) (mean - 4 
    		* sigma))) + 1;
    	int kmax = (int) Math.ceil(Math.min((double) num, 
    		(double) (mean + 4 * sigma))) - 1;
    	xLines = new Float[101];
    	yLines = new Float[101];
    	xSpikes = new Float[kmax - kmin + 1];
    	ySpikes = new Float[kmax - kmin + 1];
    	
    	for (int k = kmin; k <= kmax; k++) {
    		xSpikes[k - kmin] = new Float((k - mean) / sigma);
    		temp = (float) Combinatorics.bernoulli(num, prob, k) * sigma;
    		ySpikes[k - kmin] = new Float(temp);
	    }
	    
	    for (int i = 0; i < 101; i++) {
	    	xLines[i] = new Float(xmin + i * dx);
	    	yLines[i] = new Float(normal(xmin + i * dx, 0f, 1f));
	    }
	}
	
	// Calculate probabilities
    public void simulate(int num, float prob)
    {	
    	setPoints(num, prob);
	    
		dispArea.remove(lsg);
		lsg = new LineSpikeGraph(xLines, yLines, xSpikes, ySpikes);	// Create new LineSpikeGraph
		dispArea.add(lsg);							// Put up the graph
		validate();
	}
	
	// Calculate probabilities
    public void simulate(int num, float prob, int min, int max)
    {
    	setPoints(num, prob);
	    
		dispArea.remove(lsg);
		lsg = new LineSpikeGraph(xLines, yLines, xSpikes, ySpikes,
			((float) min - mean) / sigma, ((float) max - mean) / sigma);	// Create new LineSpikeGraph
		dispArea.add(lsg);							// Put up the graph
		
		if (min == max) {
			numl6.setText(String.valueOf(Combinatorics.bernoulli(num, prob,
				min)));
			numl8.setText(String.valueOf(normal((min - mean) / sigma, 0f,
				1f) / sigma));
		}
		else {
			float sum = 0;
			for (int k = min; k <= max; k++) {
				sum += Combinatorics.bernoulli(num, prob, k);
			}
			int flag = 1;
			numl6.setText(String.valueOf(sum));
			numl8.setText(String.valueOf(normalArea((min - mean) / sigma
				- .5f / sigma * flag, (max - mean) / sigma 
				+ .5f / sigma * flag)));
		}
		
		validate();
   	}
}




